# This file was auto-generated by Fern from our API Definition.

from .types import CountQualifier, PaginatedResponse, QualifiedCount

__all__ = ["CountQualifier", "PaginatedResponse", "QualifiedCount"]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import pydantic_v1
from ...shared_types.types.record_details import RecordDetails
from .record_references import RecordReferences


class GetRecordResponse(RecordDetails):
    """
    OK
    ---
    from sayari import GetRecordResponse, QualifiedCount, RecordReferences

    GetRecordResponse(
        id="74cf0fc2a62f9c8f4e88f8a0b3ffcca4%2FF0000110%2F1682970471254",
        label="Company Record from Hong Kong Companies Registry",
        source="74cf0fc2a62f9c8f4e88f8a0b3ffcca4",
        publication_date="2023-05-01",
        acquisition_date="2023-05-01",
        references_count=1,
        record_url="/v1/record/74cf0fc2a62f9c8f4e88f8a0b3ffcca4%2FF0000110%2F1682970471254",
        source_url="https://data.gov.hk/en-data/dataset/hk-cr-crdata-list-addr",
        document_urls=[
            "/document/74cf0fc2a62f9c8f4e88f8a0b3ffcca4%2FF0000110%2F1682970471254/file/json%2FF0000110.json"
        ],
        references=RecordReferences(
            next=False,
            offset=0,
            limit=100,
            size=QualifiedCount(
                count=1,
                qualifier="eq",
            ),
            data=[
                {
                    "entity": {
                        "id": "YUc8LtKFCpAbUIBGK8nQpw",
                        "label": "\u4e2d\u56fd\u94f6\u884c\u80a1\u4efd\u6709\u9650\u516c\u53f8",
                        "degree": 17908,
                        "entity_url": "/v1/entity/YUc8LtKFCpAbUIBGK8nQpw",
                        "pep": false,
                        "psa_count": 0,
                        "sanctioned": false,
                        "closed": false,
                        "translated_label": "Bank of China Limited",
                        "company_type": "\u4f01\u4e1a\u6cd5\u4eba",
                        "registration_date": "Incorporated 1983-10-31",
                        "latest_status": {"status": "inactive"},
                        "type": "company",
                        "identifiers": [
                            {
                                "value": "54930053HGCFWVHYZX42",
                                "type": "lei",
                                "label": "Lei",
                            },
                            {
                                "value": "911000001000013428",
                                "type": "cn_unified_social_credit_code",
                                "label": "Cn Unified Social Credit Code",
                            },
                            {
                                "value": "100000000001349",
                                "type": "cn_registration_number",
                                "label": "Cn Registration Number",
                            },
                            {
                                "value": "166044",
                                "type": "xxx_edi_global_issuer_id",
                                "label": "Xxx Edi Global Issuer Id",
                            },
                            {
                                "value": "3706001901616",
                                "type": "unknown",
                                "label": "Unknown",
                            },
                        ],
                        "addresses": [
                            "No. 1 Fuxingmen Nei Dajie, Beijing 100818, China",
                            "\u5317\u4eac\u5e02\u590d\u5174\u95e8\u5185\u5927\u88571\u53f7",
                            "1 Fuxingmen Nei Dajie, Beijing, 100818, PRC",
                        ],
                        "countries": ["CHN", "FRA", "MMR", "SGP", "HKG"],
                        "relationship_count": {
                            "linked_to": 20,
                            "branch_of": 155,
                            "has_branch": 11426,
                            "beneficial_owner_of": 1,
                            "party_to": 1,
                            "has_manager": 4,
                            "shareholder_of": 279,
                            "has_shareholder": 102,
                            "has_director": 87,
                            "has_supervisor": 28,
                            "owner_of": 3,
                            "has_subsidiary": 103,
                            "has_legal_representative": 7,
                            "issuer_of": 6006,
                        },
                        "source_count": {
                            "1fbdc8f3abdf32274f4c7657c048294a": {
                                "count": 2,
                                "label": "China CNinfo Shanghai/Shenzhen Stock Exchange Database",
                            },
                            "fcfa3d1c6b5f9744188fc01d0999fb76": {
                                "count": 404,
                                "label": "China SAIC",
                            },
                            "bf68c3a9a482c02f1f76342feb79af8d": {
                                "count": 1,
                                "label": "China Company Directory (Xin Gongshang Minglu)(Web Crawled Data)",
                            },
                            "a025b3503797a9cd1e1964dec6943594": {
                                "count": 764,
                                "label": "Hong Kong Stock Exchange - Shareholding Disclosures",
                            },
                            "78586b6984ecb00e81563ae3d31c9227": {
                                "count": 1,
                                "label": "Myanmar DICA",
                            },
                            "db3416894cd5f0c4d2d6ecc79bdaf366": {
                                "count": 21,
                                "label": "EDI Publicly-Listed Global Security Issuers (3rd Party Data)",
                            },
                            "b812677c0a32a1746b3ac741c7b97ae0": {
                                "count": 59948,
                                "label": "Legal Entity Identifier (LEI) Registry (3rd Party Data)",
                            },
                            "0ff02c63234d4447c803acd7748b9afc": {
                                "count": 15,
                                "label": "China Company Directory (Shuidi)(Web Crawled Data)",
                            },
                            "b6382672c6741fe1bca28d2668c1732b": {
                                "count": 200,
                                "label": "China LHNB MOFCOM Foreign Investment Directory",
                            },
                            "2cd05c04774a433ce4d79e61ee105d08": {
                                "count": 5,
                                "label": "Hong Kong Judiciary Judgments",
                            },
                            "0cf0044442daf17258e27a6ddd49770f": {
                                "count": 223,
                                "label": "China NECIPS",
                            },
                            "148946e7e3e5b2ba031bfcefa28e4d83": {
                                "count": 17,
                                "label": "China Company Directory (Aiqicha)(Web Crawled Data)",
                            },
                            "7e0b45866bdcca61b2cfd455e5403dc2": {
                                "count": 9,
                                "label": "China Central Government Procurement Center Enterprise Database",
                            },
                            "74cf0fc2a62f9c8f4e88f8a0b3ffcca4": {
                                "count": 2,
                                "label": "Hong Kong Companies Registry",
                            },
                            "8f21460f1cc54773b9672cb5efdb01cf": {
                                "count": 7,
                                "label": "China Ministry of Finance Government Procurement Announcements",
                            },
                        },
                        "risk": {
                            "basel_aml": {
                                "value": 6.81,
                                "metadata": {"country": ["CHN"]},
                                "level": "relevant",
                            },
                            "state_owned": {
                                "value": true,
                                "metadata": {},
                                "level": "high",
                            },
                            "meu_list_contractors": {
                                "value": true,
                                "metadata": {
                                    "sources": [
                                        "China Central Government Procurement Center Enterprise Database",
                                        "China Ministry of Finance Government Procurement Announcements",
                                    ]
                                },
                                "level": "high",
                            },
                            "cpi_score": {
                                "value": 45,
                                "metadata": {"country": ["CHN"]},
                                "level": "relevant",
                            },
                            "owner_of_forced_labor_xinjiang_entity": {
                                "value": 1,
                                "metadata": {},
                                "level": "high",
                            },
                            "owner_of_regulatory_action_entity": {
                                "value": 1,
                                "metadata": {},
                                "level": "high",
                            },
                        },
                    },
                    "type": "about",
                }
            ],
        ),
    )
    """

    references: RecordReferences

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .bucket_agg import BucketAgg
from .create_project_request import CreateProjectRequest
from .create_project_response import CreateProjectResponse
from .doc_count import DocCount
from .get_project_entities_accept_header import GetProjectEntitiesAcceptHeader
from .get_project_entities_response import GetProjectEntitiesResponse
from .get_projects_response import GetProjectsResponse
from .hs_code_agg import HsCodeAgg
from .hs_code_agg_bucket import HsCodeAggBucket
from .hs_code_agg_terms import HsCodeAggTerms
from .int_key_value import IntKeyValue
from .project import Project
from .project_counts import ProjectCounts
from .project_entities_aggs import ProjectEntitiesAggs
from .project_entities_aggs_definition import ProjectEntitiesAggsDefinition
from .project_entities_filter import ProjectEntitiesFilter
from .project_entity import ProjectEntity
from .project_entity_upstream import ProjectEntityUpstream
from .project_with_members import ProjectWithMembers
from .psa_summary import PsaSummary
from .role import Role
from .role_member import RoleMember
from .role_member_type import RoleMemberType
from .sort_field import SortField

__all__ = [
    "BucketAgg",
    "CreateProjectRequest",
    "CreateProjectResponse",
    "DocCount",
    "GetProjectEntitiesAcceptHeader",
    "GetProjectEntitiesResponse",
    "GetProjectsResponse",
    "HsCodeAgg",
    "HsCodeAggBucket",
    "HsCodeAggTerms",
    "IntKeyValue",
    "Project",
    "ProjectCounts",
    "ProjectEntitiesAggs",
    "ProjectEntitiesAggsDefinition",
    "ProjectEntitiesFilter",
    "ProjectEntity",
    "ProjectEntityUpstream",
    "ProjectWithMembers",
    "PsaSummary",
    "Role",
    "RoleMember",
    "RoleMemberType",
    "SortField",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...base_types.types.paginated_response import PaginatedResponse
from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import pydantic_v1
from .shipment import Shipment


class ShipmentSearchResponse(PaginatedResponse):
    """
    OK
    ---
    from sayari import (
        DataSource,
        QualifiedCount,
        Shipment,
        ShipmentAddress,
        ShipmentIdentifier,
        ShipmentSearchResponse,
        SourceOrDestinationEntity,
        Weight,
    )

    ShipmentSearchResponse(
        offset=0,
        limit=1,
        size=QualifiedCount(
            count=10000,
            qualifier="gte",
        ),
        next=True,
        data=[
            Shipment(
                id="KIB4wNCMtzLhG-onltADgQ",
                type="shipment",
                buyer=[
                    SourceOrDestinationEntity(
                        id="M_vGQfA6PWYdNkj5a_XMNQ",
                        names=["HEMINGWAY RUM COMPANY LLC"],
                        risks={"basel_aml": 4.63, "cpi_score": 67},
                        countries=["USA"],
                        business_purpose=[],
                    )
                ],
                supplier=[
                    SourceOrDestinationEntity(
                        id="9NQnfZhEFrRnp4YWk5yAVQ",
                        names=["FOURSQUARE RUM DISTILLERY FOURSQUARE"],
                        risks={
                            "eu_high_risk_third": true,
                            "basel_aml": 5.81,
                            "cpi_score": 65,
                        },
                        countries=["BRB"],
                        business_purpose=[],
                    )
                ],
                arrival_date="2022-05-25",
                departure_date="2022-05",
                departure_address=ShipmentAddress(
                    x=-79.4861,
                    city="ALAMANCE",
                    state="NC",
                    y=36.035,
                    value="27201",
                ),
                monetary_value=[],
                weight=[
                    Weight(
                        value=5388.0,
                        unit="kilogram",
                        type="gross_weight",
                    )
                ],
                identifier=[
                    ShipmentIdentifier(
                        value="TSCW15541208",
                        type="bill_of_lading",
                    )
                ],
                sources=[
                    DataSource(
                        id="16a4cc2d0f467fa993b28587d542a25d",
                        label="USA Imports (2021 - Present)",
                    )
                ],
                hs_codes=[],
                product_descriptions=["0000000006FOURSQUARE 5YR OLD RUM"],
                record="16a4cc2d0f467fa993b28587d542a25d/TSCW15541208/1653523200000/0",
            )
        ],
    )
    """

    offset: int
    next: bool
    data: typing.List[Shipment]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing

StatusContext = typing.Union[
    typing.Literal[
        "broker_license",
        "investment_advisor_license",
        "sole_proprietorship_status",
        "general_partnership_status",
        "limited_liability_partnership_status",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from .event_info import EventInfo
from .history_info import HistoryInfo
from .history_response import HistoryResponse
from .usage_info import UsageInfo
from .usage_response import UsageResponse

__all__ = ["EventInfo", "HistoryInfo", "HistoryResponse", "UsageInfo", "UsageResponse"]



# This file was auto-generated by Fern from our API Definition.

from .types import ResourceType, SaveEntityRequest, SaveEntityResponse, SaveEntityResponseData

__all__ = ["ResourceType", "SaveEntityRequest", "SaveEntityResponse", "SaveEntityResponseData"]



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import pydantic_v1
from ..core.remove_none_from_dict import remove_none_from_dict
from ..core.request_options import RequestOptions
from ..shared_errors.errors.bad_request import BadRequest
from ..shared_errors.errors.internal_server_error import InternalServerError
from ..shared_errors.errors.method_not_allowed import MethodNotAllowed
from ..shared_errors.errors.not_acceptable import NotAcceptable
from ..shared_errors.errors.rate_limit_exceeded import RateLimitExceeded
from ..shared_errors.errors.unauthorized import Unauthorized
from ..shared_errors.types.bad_request_response import BadRequestResponse
from ..shared_errors.types.internal_server_error_response import InternalServerErrorResponse
from ..shared_errors.types.method_not_allowed_response import MethodNotAllowedResponse
from ..shared_errors.types.not_acceptable_response import NotAcceptableResponse
from ..shared_errors.types.rate_limit_response import RateLimitResponse
from ..shared_errors.types.unauthorized_response import UnauthorizedResponse
from ..shared_types.types.search_field import SearchField
from .types.entity_search_response import EntitySearchResponse
from .types.filter_list import FilterList
from .types.record_search_response import RecordSearchResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SearchClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def search_entity(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        q: str,
        fields: typing.Optional[typing.Sequence[SearchField]] = OMIT,
        filter: typing.Optional[FilterList] = OMIT,
        facets: typing.Optional[bool] = OMIT,
        geo_facets: typing.Optional[bool] = OMIT,
        advanced: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EntitySearchResponse:
        """
        Search for an entity. Please note, searches are limited to a maximum of 10,000 results.

        Parameters:
            - limit: typing.Optional[int]. A limit on the number of objects to be returned with a range between 1 and 100. Defaults to 100.

            - offset: typing.Optional[int]. Number of results to skip before returning response. Defaults to 0.

            - q: str. Query term. The syntax for the query parameter follows elasticsearch simple query string syntax. The includes the ability to use search operators and to perform nested queries. Must be url encoded.

            - fields: typing.Optional[typing.Sequence[SearchField]]. Record or entity fields to search against.

            - filter: typing.Optional[FilterList]. Filters to be applied to search query to limit the result-set.

            - facets: typing.Optional[bool]. Whether or not to return search facets in results giving counts by field. Defaults to false.

            - geo_facets: typing.Optional[bool]. Whether or not to return search geo bound facets in results giving counts by geo tile. Defaults to false.

            - advanced: typing.Optional[bool]. Set to true to enable full elasticsearch query string syntax which allows for fielded search and more complex operators. Note that the syntax is more strict and can result in empty result-sets. Defaults to false.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.search.search_entity(
            limit=1,
            q="victoria beckham limited",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"q": q}
        if fields is not OMIT:
            _request["fields"] = fields
        if filter is not OMIT:
            _request["filter"] = filter
        if facets is not OMIT:
            _request["facets"] = facets
        if geo_facets is not OMIT:
            _request["geo_facets"] = geo_facets
        if advanced is not OMIT:
            _request["advanced"] = advanced
        _response = self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/search/entity"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "limit": limit,
                        "offset": offset,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(EntitySearchResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 406:
            raise NotAcceptable(pydantic_v1.parse_obj_as(NotAcceptableResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def search_entity_get(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        q: str,
        fields: typing.Optional[typing.Union[SearchField, typing.Sequence[SearchField]]] = None,
        facets: typing.Optional[bool] = None,
        geo_facets: typing.Optional[bool] = None,
        advanced: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EntitySearchResponse:
        """
        Search for an entity. Please note, searches are limited to a maximum of 10,000 results.

        Parameters:
            - limit: typing.Optional[int]. A limit on the number of objects to be returned with a range between 1 and 100. Defaults to 100.

            - offset: typing.Optional[int]. Number of results to skip before returning response. Defaults to 0.

            - q: str. Query term. The syntax for the query parameter follows elasticsearch simple query string syntax. The includes the ability to use search operators and to perform nested queries. Must be url encoded.

            - fields: typing.Optional[typing.Union[SearchField, typing.Sequence[SearchField]]]. Record or entity fields to search against.

            - facets: typing.Optional[bool]. Whether or not to return search facets in results giving counts by field. Defaults to false.

            - geo_facets: typing.Optional[bool]. Whether or not to return search geo bound facets in results giving counts by geo tile. Defaults to false.

            - advanced: typing.Optional[bool]. Set to true to enable full elasticsearch query string syntax which allows for fielded search and more complex operators. Note that the syntax is more strict and can result in empty result-sets. Defaults to false.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.search.search_entity_get(
            limit=1,
            q="victoria beckham limited",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/search/entity"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "limit": limit,
                        "offset": offset,
                        "q": q,
                        "fields": fields,
                        "facets": facets,
                        "geo_facets": geo_facets,
                        "advanced": advanced,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(EntitySearchResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 406:
            raise NotAcceptable(pydantic_v1.parse_obj_as(NotAcceptableResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def search_record(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        q: str,
        fields: typing.Optional[typing.Sequence[SearchField]] = OMIT,
        filter: typing.Optional[FilterList] = OMIT,
        facets: typing.Optional[bool] = OMIT,
        advanced: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RecordSearchResponse:
        """
        Search for a record. Please note, searches are limited to a maximum of 10,000 results.

        Parameters:
            - limit: typing.Optional[int]. A limit on the number of objects to be returned with a range between 1 and 100. Defaults to 100.

            - offset: typing.Optional[int]. Number of results to skip before returning response. Defaults to 0.

            - q: str. Query term. The syntax for the query parameter follows elasticsearch simple query string syntax. The includes the ability to use search operators and to perform nested queries. Must be url encoded.

            - fields: typing.Optional[typing.Sequence[SearchField]]. Record or entity fields to search against.

            - filter: typing.Optional[FilterList]. Filters to be applied to search query to limit the result-set.

            - facets: typing.Optional[bool]. Whether or not to return search facets in results giving counts by field. Defaults to false.

            - advanced: typing.Optional[bool]. Set to true to enable full elasticsearch query string syntax which allows for fielded search and more complex operators. Note that the syntax is more strict and can result in empty result-sets. Defaults to false.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.search.search_record(
            limit=1,
            q="victoria beckham limited",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"q": q}
        if fields is not OMIT:
            _request["fields"] = fields
        if filter is not OMIT:
            _request["filter"] = filter
        if facets is not OMIT:
            _request["facets"] = facets
        if advanced is not OMIT:
            _request["advanced"] = advanced
        _response = self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/search/record"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "limit": limit,
                        "offset": offset,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(RecordSearchResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 406:
            raise NotAcceptable(pydantic_v1.parse_obj_as(NotAcceptableResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def search_record_get(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        q: str,
        fields: typing.Optional[typing.Union[SearchField, typing.Sequence[SearchField]]] = None,
        facets: typing.Optional[bool] = None,
        advanced: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RecordSearchResponse:
        """
        Search for a record. Please note, searches are limited to a maximum of 10,000 results.

        Parameters:
            - limit: typing.Optional[int]. A limit on the number of objects to be returned with a range between 1 and 100. Defaults to 100.

            - offset: typing.Optional[int]. Number of results to skip before returning response. Defaults to 0.

            - q: str. Query term. The syntax for the query parameter follows elasticsearch simple query string syntax. The includes the ability to use search operators and to perform nested queries. Must be url encoded.

            - fields: typing.Optional[typing.Union[SearchField, typing.Sequence[SearchField]]]. Record or entity fields to search against.

            - facets: typing.Optional[bool]. Whether or not to return search facets in results giving counts by field. Defaults to false.

            - advanced: typing.Optional[bool]. Set to true to enable full elasticsearch query string syntax which allows for fielded search and more complex operators. Note that the syntax is more strict and can result in empty result-sets. Defaults to false.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.search.search_record_get(
            q="victoria beckham limited",
            limit=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/search/record"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "limit": limit,
                        "offset": offset,
                        "q": q,
                        "fields": fields,
                        "facets": facets,
                        "advanced": advanced,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(RecordSearchResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 406:
            raise NotAcceptable(pydantic_v1.parse_obj_as(NotAcceptableResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncSearchClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def search_entity(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        q: str,
        fields: typing.Optional[typing.Sequence[SearchField]] = OMIT,
        filter: typing.Optional[FilterList] = OMIT,
        facets: typing.Optional[bool] = OMIT,
        geo_facets: typing.Optional[bool] = OMIT,
        advanced: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EntitySearchResponse:
        """
        Search for an entity. Please note, searches are limited to a maximum of 10,000 results.

        Parameters:
            - limit: typing.Optional[int]. A limit on the number of objects to be returned with a range between 1 and 100. Defaults to 100.

            - offset: typing.Optional[int]. Number of results to skip before returning response. Defaults to 0.

            - q: str. Query term. The syntax for the query parameter follows elasticsearch simple query string syntax. The includes the ability to use search operators and to perform nested queries. Must be url encoded.

            - fields: typing.Optional[typing.Sequence[SearchField]]. Record or entity fields to search against.

            - filter: typing.Optional[FilterList]. Filters to be applied to search query to limit the result-set.

            - facets: typing.Optional[bool]. Whether or not to return search facets in results giving counts by field. Defaults to false.

            - geo_facets: typing.Optional[bool]. Whether or not to return search geo bound facets in results giving counts by geo tile. Defaults to false.

            - advanced: typing.Optional[bool]. Set to true to enable full elasticsearch query string syntax which allows for fielded search and more complex operators. Note that the syntax is more strict and can result in empty result-sets. Defaults to false.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.search.search_entity(
            limit=1,
            q="victoria beckham limited",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"q": q}
        if fields is not OMIT:
            _request["fields"] = fields
        if filter is not OMIT:
            _request["filter"] = filter
        if facets is not OMIT:
            _request["facets"] = facets
        if geo_facets is not OMIT:
            _request["geo_facets"] = geo_facets
        if advanced is not OMIT:
            _request["advanced"] = advanced
        _response = await self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/search/entity"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "limit": limit,
                        "offset": offset,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(EntitySearchResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 406:
            raise NotAcceptable(pydantic_v1.parse_obj_as(NotAcceptableResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def search_entity_get(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        q: str,
        fields: typing.Optional[typing.Union[SearchField, typing.Sequence[SearchField]]] = None,
        facets: typing.Optional[bool] = None,
        geo_facets: typing.Optional[bool] = None,
        advanced: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EntitySearchResponse:
        """
        Search for an entity. Please note, searches are limited to a maximum of 10,000 results.

        Parameters:
            - limit: typing.Optional[int]. A limit on the number of objects to be returned with a range between 1 and 100. Defaults to 100.

            - offset: typing.Optional[int]. Number of results to skip before returning response. Defaults to 0.

            - q: str. Query term. The syntax for the query parameter follows elasticsearch simple query string syntax. The includes the ability to use search operators and to perform nested queries. Must be url encoded.

            - fields: typing.Optional[typing.Union[SearchField, typing.Sequence[SearchField]]]. Record or entity fields to search against.

            - facets: typing.Optional[bool]. Whether or not to return search facets in results giving counts by field. Defaults to false.

            - geo_facets: typing.Optional[bool]. Whether or not to return search geo bound facets in results giving counts by geo tile. Defaults to false.

            - advanced: typing.Optional[bool]. Set to true to enable full elasticsearch query string syntax which allows for fielded search and more complex operators. Note that the syntax is more strict and can result in empty result-sets. Defaults to false.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.search.search_entity_get(
            limit=1,
            q="victoria beckham limited",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/search/entity"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "limit": limit,
                        "offset": offset,
                        "q": q,
                        "fields": fields,
                        "facets": facets,
                        "geo_facets": geo_facets,
                        "advanced": advanced,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(EntitySearchResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 406:
            raise NotAcceptable(pydantic_v1.parse_obj_as(NotAcceptableResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def search_record(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        q: str,
        fields: typing.Optional[typing.Sequence[SearchField]] = OMIT,
        filter: typing.Optional[FilterList] = OMIT,
        facets: typing.Optional[bool] = OMIT,
        advanced: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RecordSearchResponse:
        """
        Search for a record. Please note, searches are limited to a maximum of 10,000 results.

        Parameters:
            - limit: typing.Optional[int]. A limit on the number of objects to be returned with a range between 1 and 100. Defaults to 100.

            - offset: typing.Optional[int]. Number of results to skip before returning response. Defaults to 0.

            - q: str. Query term. The syntax for the query parameter follows elasticsearch simple query string syntax. The includes the ability to use search operators and to perform nested queries. Must be url encoded.

            - fields: typing.Optional[typing.Sequence[SearchField]]. Record or entity fields to search against.

            - filter: typing.Optional[FilterList]. Filters to be applied to search query to limit the result-set.

            - facets: typing.Optional[bool]. Whether or not to return search facets in results giving counts by field. Defaults to false.

            - advanced: typing.Optional[bool]. Set to true to enable full elasticsearch query string syntax which allows for fielded search and more complex operators. Note that the syntax is more strict and can result in empty result-sets. Defaults to false.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.search.search_record(
            limit=1,
            q="victoria beckham limited",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"q": q}
        if fields is not OMIT:
            _request["fields"] = fields
        if filter is not OMIT:
            _request["filter"] = filter
        if facets is not OMIT:
            _request["facets"] = facets
        if advanced is not OMIT:
            _request["advanced"] = advanced
        _response = await self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/search/record"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "limit": limit,
                        "offset": offset,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(RecordSearchResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 406:
            raise NotAcceptable(pydantic_v1.parse_obj_as(NotAcceptableResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def search_record_get(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        q: str,
        fields: typing.Optional[typing.Union[SearchField, typing.Sequence[SearchField]]] = None,
        facets: typing.Optional[bool] = None,
        advanced: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RecordSearchResponse:
        """
        Search for a record. Please note, searches are limited to a maximum of 10,000 results.

        Parameters:
            - limit: typing.Optional[int]. A limit on the number of objects to be returned with a range between 1 and 100. Defaults to 100.

            - offset: typing.Optional[int]. Number of results to skip before returning response. Defaults to 0.

            - q: str. Query term. The syntax for the query parameter follows elasticsearch simple query string syntax. The includes the ability to use search operators and to perform nested queries. Must be url encoded.

            - fields: typing.Optional[typing.Union[SearchField, typing.Sequence[SearchField]]]. Record or entity fields to search against.

            - facets: typing.Optional[bool]. Whether or not to return search facets in results giving counts by field. Defaults to false.

            - advanced: typing.Optional[bool]. Set to true to enable full elasticsearch query string syntax which allows for fielded search and more complex operators. Note that the syntax is more strict and can result in empty result-sets. Defaults to false.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.search.search_record_get(
            q="victoria beckham limited",
            limit=1,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/search/record"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "limit": limit,
                        "offset": offset,
                        "q": q,
                        "fields": fields,
                        "facets": facets,
                        "advanced": advanced,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(RecordSearchResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 406:
            raise NotAcceptable(pydantic_v1.parse_obj_as(NotAcceptableResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ...core.api_error import ApiError
from ..types.not_acceptable_response import NotAcceptableResponse


class NotAcceptable(ApiError):
    def __init__(self, body: NotAcceptableResponse):
        super().__init__(status_code=406, body=body)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import pydantic_v1
from ...generated_types.types.attribute_details import AttributeDetails
from .company_type import CompanyType
from .embedded_entity import EmbeddedEntity
from .entity_hs_code import EntityHsCode
from .entity_registration_date import EntityRegistrationDate
from .entity_risk import EntityRisk
from .entity_translated_label import EntityTranslatedLabel
from .possibly_same_as import PossiblySameAs
from .referenced_by import ReferencedBy
from .shipment_arrival import ShipmentArrival
from .shipment_departure import ShipmentDeparture
from .status import Status


class EntityDetails(EmbeddedEntity):
    """
    Additional fields providing more details about an entity
    """

    registration_date: typing.Optional[EntityRegistrationDate] = None
    translated_label: typing.Optional[EntityTranslatedLabel] = None
    hs_code: typing.Optional[EntityHsCode] = None
    shipment_arrival: typing.Optional[ShipmentArrival] = None
    shipment_departure: typing.Optional[ShipmentDeparture] = None
    company_type: typing.Optional[CompanyType] = None
    latest_status: typing.Optional[Status] = None
    risk: EntityRisk = pydantic_v1.Field()
    """
    [Risk factors](/sayari-library/ontology/risk-factors) associated with the entity.
    """

    attributes: typing.Optional[AttributeDetails] = pydantic_v1.Field(default=None)
    """
    Detailed information about the entity's [attributes](/sayari-library/ontology/attributes).
    """

    relationships: typing.Optional[EntityRelationships] = pydantic_v1.Field(default=None)
    """
    Detailed information about the entity's [relationships](/sayari-library/ontology/relationships).
    """

    possibly_same_as: typing.Optional[PossiblySameAs] = None
    referenced_by: typing.Optional[ReferencedBy] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}


from .entity_relationships import EntityRelationships  # noqa: E402

EntityDetails.update_forward_refs()



# This file was auto-generated by Fern from our API Definition.

import typing

SortField = typing.Union[
    typing.Literal[
        "+created",
        "-created",
        "+saved_resource_id",
        "-saved_resource_id",
        "+entity_id",
        "-entity_id",
        "+country",
        "-country",
        "+latest_status",
        "-latest_status",
        "+label",
        "-label",
        "+receiver_of",
        "-receiver_of",
        "+shipper_of",
        "-shipper_of",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import pydantic_v1
from .attribute_response_data import AttributeResponseData


class AttributeResponse(pydantic_v1.BaseModel):
    """
    OK
    ---
    from sayari import AttributeProperties, AttributeResponse, AttributeResponseData

    AttributeResponse(
        data=AttributeResponseData(
            value={
                "street1": "1600 Pennsylvania Avenue NW",
                "city": "Washington DC",
                "state": "Washington DC",
                "postalCode": "20500",
                "country": "US",
            },
            properties=[
                AttributeProperties(
                    editable=True,
                    record_count=0,
                    id="enEwNGF4WDJkTG45dEU2VzZROFFoZ3xhZGRyZXNzfDBwbEVCMHxVNzhzN21yOUVFTThIZ3pwREM3UDFB",
                )
            ],
        ),
    )
    """

    data: AttributeResponseData

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import pydantic_v1
from .resource_type import ResourceType


class SaveEntityRequest(pydantic_v1.BaseModel):
    """
    from sayari import SaveEntityRequest

    SaveEntityRequest(
        type="entity",
        project="GNJbkG",
        entity_id="Zk0qOaM2SSYg_ZhsljykMQ",
    )
    """

    type: ResourceType
    project: str = pydantic_v1.Field()
    """
    The project identifier.
    """

    entity_id: str = pydantic_v1.Field()
    """
    The entity identifier.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import pydantic_v1


class GenericProperties(pydantic_v1.BaseModel):
    date: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    as-of date
    """

    from_date: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    start date
    """

    to_date: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    end date
    """

    type: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    A text description of the attribute (e.g., "name of pet")
    """

    value: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    The value of the attribute as text (e.g., "Max")
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .match_explanation import MatchExplanation
from .resolution_response import ResolutionResponse
from .resolution_response_fields import ResolutionResponseFields
from .resolution_result import ResolutionResult

__all__ = ["MatchExplanation", "ResolutionResponse", "ResolutionResponseFields", "ResolutionResult"]



# This file was auto-generated by Fern from our API Definition.

import typing

WeakIdentifierType = typing.Union[
    typing.Literal[
        "unknown",
        "mx_partial_rfc_person",
        "ru_oktmo",
        "ru_kpp",
        "ru_okpo",
        "unknown_passport",
        "rks_kta_number",
        "bra_partial_cpf",
        "ven_colegiado_number",
        "pan_folio",
        "kgz_okpo",
        "kgz_inn",
        "lbn_registration_number",
        "ita_ebr_reg_number",
        "ita_ebr_short_search_code",
        "chl_santiago_gazette_cve",
        "bmu_registration_number",
        "brazilian_oab",
        "irn_reg_number",
        "mx_fme",
        "ukr_edrpou",
        "ru_license_number",
        "maritime_call_sign",
        "prk_ship_reg_no",
        "cofi_code",
        "ru_nza",
        "arg_igj_number",
        "unknown_bra_case_number",
        "irq_provision_card",
        "mbl_hm_sn",
        "mbl_hm_hud_num",
        "mbl_hm_title_num",
        "mex_declaranet_acuse",
        "deu_registernummer",
        "unknown_national_id_num",
        "unknown_civil_reg_num",
        "unknown_residency_num",
        "unknown_folio_id_num",
        "unknown_commercial_register_id",
        "unknown_chamber_of_commerce_id",
        "unknown_license_num",
        "unknown_industrial_license_num",
        "cze_file_number",
        "mex_tm_app_no",
        "mex_tm_reg_no",
        "jordan_company_no",
        "cn_qcc_internal_id",
        "jor_sol_prop_institution_number",
        "usa_nc_corp_no",
        "usa_nm_license_id",
        "khm_tin_number",
        "usa_mo_entity_id",
        "usa_mo_corp_number",
        "mac_raem_case_number",
        "hnd_tegucigalpa_notary",
        "lbn_family_number",
        "tx_bexar_property_geo_id",
        "pak_cnic_family_no",
        "rou_commercial_register_id",
        "south_africa_partial_id_number",
        "prk_internal_trade_id",
        "chn_customs_registration_code",
        "aut_former_cr_no",
        "aut_natl_bank_no",
        "lva_person_id_masked",
        "lva_court_case_id",
        "chn_cninfo_legal_person_id",
        "rks_business_number",
        "rks_fiscal_number",
        "mdg_nif_number",
        "mdg_rcs_number",
        "vat",
        "usa_il_chicago_site_number",
        "usa_generic_ticker",
        "ven_rnc_number",
        "usa_imports_system_identity_id",
        "col_bill_of_lading",
        "col_secop_no",
        "pol_regon_number",
        "pol_nip_number",
        "bill_of_lading",
        "pan_ibc_ruc",
        "pak_old_company_code",
        "jpn_permit_no",
        "dma_corporate_registry_entity_num",
        "atg_corporate_registry_entity_num",
        "lca_corporate_registry_entity_num",
        "ven_manifiesto_number",
        "col_matricula_mercantil",
        "maritime_mobile_service_identity",
        "usa_former_fein",
        "usa_cbp_wro_id",
        "ccs_ship_class_number",
        "tur_partial_mersis_number",
        "tur_office_registration_number",
        "partial_ven_cedula",
        "rus_cbr_id",
        "geo_state_registration_number",
        "bih_mbs_number",
        "gbr_grant_info_number",
        "pry_shipment_number",
        "arg_import_export_id",
        "cri_op_no",
        "aus_state_reg_number",
        "hun_opten_id",
        "malformed mmr_prior_reg_no",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from .types import (
    ClientName,
    CompanyType,
    Coordinate,
    CoreEntity,
    EmbeddedEntity,
    EntityDetails,
    EntityHsCode,
    EntityMatches,
    EntityRegistrationDate,
    EntityRelationships,
    EntityRisk,
    EntitySummary,
    EntityTranslatedLabel,
    Identifier,
    PossiblySameAs,
    PossiblySameAsData,
    PossiblySameAsMatch,
    Psa,
    PsaEntity,
    PsaMatchKeys,
    RecordDetails,
    ReferencedBy,
    ReferencedByData,
    ReferencedByDataType,
    RelationshipCount,
    RelationshipData,
    RelationshipInfo,
    RiskData,
    RiskLevel,
    RiskValue,
    SearchField,
    ShipmentArrival,
    ShipmentDeparture,
    SourceCountInfo,
    Status,
)

__all__ = [
    "ClientName",
    "CompanyType",
    "Coordinate",
    "CoreEntity",
    "EmbeddedEntity",
    "EntityDetails",
    "EntityHsCode",
    "EntityMatches",
    "EntityRegistrationDate",
    "EntityRelationships",
    "EntityRisk",
    "EntitySummary",
    "EntityTranslatedLabel",
    "Identifier",
    "PossiblySameAs",
    "PossiblySameAsData",
    "PossiblySameAsMatch",
    "Psa",
    "PsaEntity",
    "PsaMatchKeys",
    "RecordDetails",
    "ReferencedBy",
    "ReferencedByData",
    "ReferencedByDataType",
    "RelationshipCount",
    "RelationshipData",
    "RelationshipInfo",
    "RiskData",
    "RiskLevel",
    "RiskValue",
    "SearchField",
    "ShipmentArrival",
    "ShipmentDeparture",
    "SourceCountInfo",
    "Status",
]



# This file was auto-generated by Fern from our API Definition.

import typing

NameContext = typing.Union[
    typing.Literal["primary", "given", "family", "alias", "machine_translation", "transliteration"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import pydantic_v1
from .project_with_members import ProjectWithMembers


class GetProjectsResponse(pydantic_v1.BaseModel):
    """
    from sayari import (
        GetProjectsResponse,
        ProjectCounts,
        ProjectWithMembers,
        RoleMember,
    )

    GetProjectsResponse(
        prev="MjAyMy0xMC0yNSAxNDo0NDowNi4zMjIxMTcrMDB8VjAzTU5Z",
        next="MjAyMy0wOS0xNSAxODoyNDozOC45ODEwMjMrMDB8OFlWQjZZ",
        limit=8,
        data=[
            ProjectWithMembers(
                id="V03MNY",
                label="Project 1",
                archived=False,
                created="2023-10-25 14:44:06.322117+00",
                updated="2023-10-25 14:44:06.322117+00",
                counts=ProjectCounts(),
                members=[
                    RoleMember(
                        type="user",
                        id="auth0|7a8f3e2b91d476c5b2e04a87",
                        role="admin",
                        created="2023-10-25T14:44:06.322117+00:00",
                        updated="2023-10-25T14:44:06.322117+00:00",
                    )
                ],
            ),
            ProjectWithMembers(
                id="eYDDmY",
                label="Project 2",
                archived=False,
                created="2023-10-24 20:41:21.235451+00",
                updated="2023-10-24 20:41:21.235451+00",
                counts=ProjectCounts(
                    graph=1,
                    entity=2530,
                ),
                members=[
                    RoleMember(
                        type="user",
                        id="auth0|f0bc63a9e72d18ef4c5702d6",
                        role="admin",
                        created="2023-10-24T20:41:21.235451+00:00",
                        updated="2023-10-24T20:41:21.235451+00:00",
                    ),
                    RoleMember(
                        type="group",
                        id="org_VdFgkL2qNpweRZAs",
                        role="viewer",
                        created="2023-11-04T18:40:30.942863+00:00",
                        updated="2023-11-04T18:40:30.942863+00:00",
                    ),
                ],
            ),
        ],
    )
    """

    next: typing.Optional[str] = None
    prev: typing.Optional[str] = None
    first: typing.Optional[bool] = None
    last: typing.Optional[bool] = None
    limit: int
    data: typing.List[ProjectWithMembers]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import pydantic_v1
from .currency import Currency


class SharesProperties(pydantic_v1.BaseModel):
    currency: typing.Optional[Currency] = pydantic_v1.Field(default=None)
    """
    The currency of the monetary_value
    """

    date: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    as-of date
    """

    from_date: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    start date
    """

    monetary_value: typing.Optional[float] = pydantic_v1.Field(default=None)
    """
    The total monetary value of the shares
    """

    num_shares: typing.Optional[float] = pydantic_v1.Field(default=None)
    """
    The number of shares held, issued, etc.
    """

    percentage: typing.Optional[float] = pydantic_v1.Field(default=None)
    """
    The percentage of shares owned
    """

    to_date: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    end date
    """

    type: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    A string describing the type of shares (e.g., "Class B", "Protected cell shares")
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import pydantic_v1
from ..core.remove_none_from_dict import remove_none_from_dict
from ..core.request_options import RequestOptions
from ..generated_types.types.both_identifier_types import BothIdentifierTypes
from ..generated_types.types.country import Country
from ..generated_types.types.entities import Entities
from ..shared_errors.errors.bad_request import BadRequest
from ..shared_errors.errors.internal_server_error import InternalServerError
from ..shared_errors.errors.method_not_allowed import MethodNotAllowed
from ..shared_errors.errors.not_acceptable import NotAcceptable
from ..shared_errors.errors.rate_limit_exceeded import RateLimitExceeded
from ..shared_errors.errors.unauthorized import Unauthorized
from ..shared_errors.types.bad_request_response import BadRequestResponse
from ..shared_errors.types.internal_server_error_response import InternalServerErrorResponse
from ..shared_errors.types.method_not_allowed_response import MethodNotAllowedResponse
from ..shared_errors.types.not_acceptable_response import NotAcceptableResponse
from ..shared_errors.types.rate_limit_response import RateLimitResponse
from ..shared_errors.types.unauthorized_response import UnauthorizedResponse
from .types.resolution_response import ResolutionResponse


class ResolutionClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def resolution(
        self,
        *,
        name: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        identifier: typing.Optional[typing.Union[BothIdentifierTypes, typing.Sequence[BothIdentifierTypes]]] = None,
        country: typing.Optional[typing.Union[Country, typing.Sequence[Country]]] = None,
        address: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        date_of_birth: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        contact: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        type: typing.Optional[typing.Union[Entities, typing.Sequence[Entities]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResolutionResponse:
        """
        The resolution endpoints allow users to search for matching entities against a provided list of attributes. The endpoint is similar to the search endpoint, except it's tuned to only return the best match so the client doesn't need to do as much or any post-processing work to filter down results.

        Parameters:
            - name: typing.Optional[typing.Union[str, typing.Sequence[str]]]. Entity name

            - identifier: typing.Optional[typing.Union[BothIdentifierTypes, typing.Sequence[BothIdentifierTypes]]]. Entity identifier. Can be from either the [Identifier Type](/sayari-library/ontology/enumerated-types#identifier-type) or [Weak Identifier Type](/sayari-library/ontology/enumerated-types#weak-identifier-type) enums.

            - country: typing.Optional[typing.Union[Country, typing.Sequence[Country]]]. Entity country - must be ISO (3166) Trigram i.e., `USA`. See complete list [here](/sayari-library/ontology/enumerated-types#country)

            - address: typing.Optional[typing.Union[str, typing.Sequence[str]]]. Entity address

            - date_of_birth: typing.Optional[typing.Union[str, typing.Sequence[str]]]. Entity date of birth

            - contact: typing.Optional[typing.Union[str, typing.Sequence[str]]]. Entity contact

            - type: typing.Optional[typing.Union[Entities, typing.Sequence[Entities]]]. [Entity type](/sayari-library/ontology/entities). If multiple values are passed for any field, the endpoint will match entities with ANY of the values.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.resolution.resolution(
            name="victoria beckham limited",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/resolution"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "name": name,
                        "identifier": identifier,
                        "country": country,
                        "address": address,
                        "date_of_birth": date_of_birth,
                        "contact": contact,
                        "type": type,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ResolutionResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 406:
            raise NotAcceptable(pydantic_v1.parse_obj_as(NotAcceptableResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncResolutionClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def resolution(
        self,
        *,
        name: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        identifier: typing.Optional[typing.Union[BothIdentifierTypes, typing.Sequence[BothIdentifierTypes]]] = None,
        country: typing.Optional[typing.Union[Country, typing.Sequence[Country]]] = None,
        address: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        date_of_birth: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        contact: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        type: typing.Optional[typing.Union[Entities, typing.Sequence[Entities]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResolutionResponse:
        """
        The resolution endpoints allow users to search for matching entities against a provided list of attributes. The endpoint is similar to the search endpoint, except it's tuned to only return the best match so the client doesn't need to do as much or any post-processing work to filter down results.

        Parameters:
            - name: typing.Optional[typing.Union[str, typing.Sequence[str]]]. Entity name

            - identifier: typing.Optional[typing.Union[BothIdentifierTypes, typing.Sequence[BothIdentifierTypes]]]. Entity identifier. Can be from either the [Identifier Type](/sayari-library/ontology/enumerated-types#identifier-type) or [Weak Identifier Type](/sayari-library/ontology/enumerated-types#weak-identifier-type) enums.

            - country: typing.Optional[typing.Union[Country, typing.Sequence[Country]]]. Entity country - must be ISO (3166) Trigram i.e., `USA`. See complete list [here](/sayari-library/ontology/enumerated-types#country)

            - address: typing.Optional[typing.Union[str, typing.Sequence[str]]]. Entity address

            - date_of_birth: typing.Optional[typing.Union[str, typing.Sequence[str]]]. Entity date of birth

            - contact: typing.Optional[typing.Union[str, typing.Sequence[str]]]. Entity contact

            - type: typing.Optional[typing.Union[Entities, typing.Sequence[Entities]]]. [Entity type](/sayari-library/ontology/entities). If multiple values are passed for any field, the endpoint will match entities with ANY of the values.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.resolution.resolution(
            name="victoria beckham limited",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/resolution"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "name": name,
                        "identifier": identifier,
                        "country": country,
                        "address": address,
                        "date_of_birth": date_of_birth,
                        "contact": contact,
                        "type": type,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ResolutionResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 406:
            raise NotAcceptable(pydantic_v1.parse_obj_as(NotAcceptableResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import pydantic_v1
from .shortest_path_data import ShortestPathData


class ShortestPathResponse(pydantic_v1.BaseModel):
    """
    OK
    ---
    from sayari import (
        EntityDetails,
        Identifier,
        RelationshipInfo,
        ShortestPathData,
        ShortestPathResponse,
        SourceCountInfo,
        Status,
        TraversalPath,
        TraversalRelationshipData,
    )

    ShortestPathResponse(
        entities=["mGq1lpuqKssNWTjIokuPeA", "ekPP386sHXHREr15-iVgvw"],
        data=[
            ShortestPathData(
                source="mGq1lpuqKssNWTjIokuPeA",
                target=EntityDetails(
                    id="ekPP386sHXHREr15-iVgvw",
                    label="BECKHAM RETAIL LIMITED",
                    degree=6,
                    entity_url="/v1/entity/ekPP386sHXHREr15-iVgvw",
                    pep=False,
                    psa_count=0,
                    sanctioned=False,
                    closed=False,
                    company_type="Private Limited Company",
                    registration_date="Incorporated 2013-08-22",
                    latest_status=Status(
                        status="active",
                    ),
                    type="company",
                    identifiers=[
                        Identifier(
                            value="08661308",
                            type="uk_company_number",
                            label="Uk Company Number",
                        )
                    ],
                    addresses=["202 HAMMERSMITH ROAD, LONDON, W6 7DN"],
                    countries=["GBR"],
                    relationship_count={
                        "has_director": 4,
                        "has_shareholder": 1,
                        "linked_to": 1,
                        "has_registered_agent": 1,
                    },
                    source_count={
                        "ecdfb3f2ecc8c3797e77d5795a8066ef": SourceCountInfo(
                            count=32,
                            label="UK Corporate Registry",
                        ),
                        "2b618f1996252fe537a6d998ae14c9b2": SourceCountInfo(
                            count=1,
                            label="UK Corporate Registry Confirmation Statements",
                        ),
                        "4ea8bac1bed868e1510ffd21842e9551": SourceCountInfo(
                            count=18,
                            label="UK Persons with Significant Control",
                        ),
                    },
                    risk={},
                    user_attribute_count={},
                    user_record_count=0,
                    user_related_entities_count=0,
                    user_relationship_count={},
                    related_entities_count=6,
                    attribute_count={
                        "company_type": 1,
                        "name": 2,
                        "business_purpose": 2,
                        "identifier": 1,
                        "country": 2,
                        "status": 2,
                        "address": 1,
                    },
                ),
                path=[
                    TraversalPath(
                        field="shareholder_of",
                        entity=EntityDetails(
                            id="ekPP386sHXHREr15-iVgvw",
                            label="BECKHAM RETAIL LIMITED",
                            degree=6,
                            entity_url="/v1/entity/ekPP386sHXHREr15-iVgvw",
                            pep=False,
                            psa_count=0,
                            sanctioned=False,
                            closed=False,
                            company_type="Private Limited Company",
                            registration_date="Incorporated 2013-08-22",
                            latest_status=Status(
                                status="active",
                            ),
                            type="company",
                            identifiers=[
                                Identifier(
                                    value="08661308",
                                    type="uk_company_number",
                                    label="Uk Company Number",
                                )
                            ],
                            addresses=["202 HAMMERSMITH ROAD, LONDON, W6 7DN"],
                            countries=["GBR"],
                            relationship_count={
                                "has_director": 4,
                                "has_shareholder": 1,
                                "linked_to": 1,
                                "has_registered_agent": 1,
                            },
                            source_count={
                                "ecdfb3f2ecc8c3797e77d5795a8066ef": SourceCountInfo(
                                    count=32,
                                    label="UK Corporate Registry",
                                ),
                                "2b618f1996252fe537a6d998ae14c9b2": SourceCountInfo(
                                    count=1,
                                    label="UK Corporate Registry Confirmation Statements",
                                ),
                                "4ea8bac1bed868e1510ffd21842e9551": SourceCountInfo(
                                    count=18,
                                    label="UK Persons with Significant Control",
                                ),
                            },
                            risk={},
                            user_attribute_count={},
                            user_record_count=0,
                            user_related_entities_count=0,
                            user_relationship_count={},
                            related_entities_count=6,
                            attribute_count={
                                "company_type": 1,
                                "name": 2,
                                "business_purpose": 2,
                                "identifier": 1,
                                "country": 2,
                                "status": 2,
                                "address": 1,
                            },
                        ),
                        relationships={
                            "linked_to": TraversalRelationshipData(
                                values=[
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/08661308/1560176240192",
                                        acquisition_date="2019-06-10",
                                        attributes={
                                            "position": [
                                                {
                                                    "value": "Has right to appoint and remove directors"
                                                }
                                            ]
                                        },
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1586822400000",
                                        acquisition_date="2020-04-14",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1601424000000",
                                        acquisition_date="2020-09-30",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1616630400000",
                                        acquisition_date="2021-03-25",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1634169600000",
                                        acquisition_date="2021-10-14",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1649894400000",
                                        acquisition_date="2022-04-14",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1667433600000",
                                        acquisition_date="2022-11-03",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1675123200000",
                                        acquisition_date="2023-01-31",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1676505600000",
                                        acquisition_date="2023-02-16",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1678320000000",
                                        acquisition_date="2023-03-09",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1680566400000",
                                        acquisition_date="2023-04-04",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1682899200000",
                                        acquisition_date="2023-05-01",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1685577600000",
                                        acquisition_date="2023-06-01",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1689033600000",
                                        acquisition_date="2023-07-11",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1692662400000",
                                        acquisition_date="2023-08-22",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1694476800000",
                                        acquisition_date="2023-09-12",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1696896000000",
                                        acquisition_date="2023-10-10",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1699488000000",
                                        acquisition_date="2023-11-09",
                                        attributes={},
                                    ),
                                ],
                            ),
                            "shareholder_of": TraversalRelationshipData(
                                values=[
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/08661308/1560176240192",
                                        acquisition_date="2019-06-10",
                                        attributes={
                                            "position": [
                                                {
                                                    "value": "Has right to appoint and remove directors"
                                                }
                                            ]
                                        },
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1586822400000",
                                        acquisition_date="2020-04-14",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1601424000000",
                                        acquisition_date="2020-09-30",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1616630400000",
                                        acquisition_date="2021-03-25",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1634169600000",
                                        acquisition_date="2021-10-14",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1649894400000",
                                        acquisition_date="2022-04-14",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1667433600000",
                                        acquisition_date="2022-11-03",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1675123200000",
                                        acquisition_date="2023-01-31",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1676505600000",
                                        acquisition_date="2023-02-16",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1678320000000",
                                        acquisition_date="2023-03-09",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1680566400000",
                                        acquisition_date="2023-04-04",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1682899200000",
                                        acquisition_date="2023-05-01",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1685577600000",
                                        acquisition_date="2023-06-01",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1689033600000",
                                        acquisition_date="2023-07-11",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1692662400000",
                                        acquisition_date="2023-08-22",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1694476800000",
                                        acquisition_date="2023-09-12",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1696896000000",
                                        acquisition_date="2023-10-10",
                                        attributes={},
                                    ),
                                    RelationshipInfo(
                                        record="4ea8bac1bed868e1510ffd21842e9551/ef8cbb3fcc71d09f098f00728df231e8/1699488000000",
                                        acquisition_date="2023-11-09",
                                        attributes={},
                                    ),
                                ],
                            ),
                        },
                    )
                ],
            )
        ],
    )
    """

    entities: typing.List[str]
    data: typing.List[ShortestPathData]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import pydantic_v1
from ..core.remove_none_from_dict import remove_none_from_dict
from ..core.request_options import RequestOptions
from ..generated_types.types.country import Country
from ..generated_types.types.relationships import Relationships
from ..generated_types.types.risk import Risk
from ..shared_errors.errors.bad_request import BadRequest
from ..shared_errors.errors.internal_server_error import InternalServerError
from ..shared_errors.errors.method_not_allowed import MethodNotAllowed
from ..shared_errors.errors.not_found import NotFound
from ..shared_errors.errors.rate_limit_exceeded import RateLimitExceeded
from ..shared_errors.errors.unauthorized import Unauthorized
from ..shared_errors.types.bad_request_response import BadRequestResponse
from ..shared_errors.types.internal_server_error_response import InternalServerErrorResponse
from ..shared_errors.types.method_not_allowed_response import MethodNotAllowedResponse
from ..shared_errors.types.not_found_response import NotFoundResponse
from ..shared_errors.types.rate_limit_response import RateLimitResponse
from ..shared_errors.types.unauthorized_response import UnauthorizedResponse
from .types.entity_summary_response import EntitySummaryResponse
from .types.get_entity_response import GetEntityResponse


class EntityClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_entity(
        self,
        id: str,
        *,
        attributes_name_next: typing.Optional[str] = None,
        attributes_name_prev: typing.Optional[str] = None,
        attributes_name_limit: typing.Optional[int] = None,
        attributes_address_next: typing.Optional[str] = None,
        attributes_address_prev: typing.Optional[str] = None,
        attributes_address_limit: typing.Optional[int] = None,
        attributes_country_next: typing.Optional[str] = None,
        attributes_country_prev: typing.Optional[str] = None,
        attributes_country_limit: typing.Optional[int] = None,
        relationships_next: typing.Optional[str] = None,
        relationships_prev: typing.Optional[str] = None,
        relationships_limit: typing.Optional[int] = None,
        relationships_type: typing.Optional[Relationships] = None,
        relationships_sort: typing.Optional[str] = None,
        relationships_start_date: typing.Optional[dt.date] = None,
        relationships_end_date: typing.Optional[dt.date] = None,
        relationships_min_shares: typing.Optional[int] = None,
        relationships_country: typing.Optional[typing.Union[Country, typing.Sequence[Country]]] = None,
        relationships_arrival_country: typing.Optional[typing.Union[Country, typing.Sequence[Country]]] = None,
        relationships_arrival_state: typing.Optional[str] = None,
        relationships_arrival_city: typing.Optional[str] = None,
        relationships_departure_country: typing.Optional[typing.Union[Country, typing.Sequence[Country]]] = None,
        relationships_departure_state: typing.Optional[str] = None,
        relationships_departure_city: typing.Optional[str] = None,
        relationships_partner_name: typing.Optional[str] = None,
        relationships_partner_risk: typing.Optional[typing.Union[Risk, typing.Sequence[Risk]]] = None,
        relationships_hs_code: typing.Optional[str] = None,
        possibly_same_as_next: typing.Optional[str] = None,
        possibly_same_as_prev: typing.Optional[str] = None,
        possibly_same_as_limit: typing.Optional[int] = None,
        referenced_by_next: typing.Optional[str] = None,
        referenced_by_prev: typing.Optional[str] = None,
        referenced_by_limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetEntityResponse:
        """
        Retrieve an entity from the database based on the ID

        Parameters:
            - id: str. Unique identifier of the entity

            - attributes_name_next: typing.Optional[str]. The pagination token for the next page of attribute `name`.

            - attributes_name_prev: typing.Optional[str]. The pagination token for the previous page of attribute `name`.

            - attributes_name_limit: typing.Optional[int]. Limit total values returned for attribute `name`. Defaults to 100.

            - attributes_address_next: typing.Optional[str]. The pagination token for the next page of attribute `address`.

            - attributes_address_prev: typing.Optional[str]. The pagination token for the previous page of attribute `address`.

            - attributes_address_limit: typing.Optional[int]. Limit total values returned for attribute `address`. Defaults to 100.

            - attributes_country_next: typing.Optional[str]. The pagination token for the next page of attribute `country`.

            - attributes_country_prev: typing.Optional[str]. The pagination token for the previous page of attribute `country`.

            - attributes_country_limit: typing.Optional[int]. Limit total values returned for attribute `country`. Defaults to 100.

            - relationships_next: typing.Optional[str]. The pagination token for the next page of relationship results

            - relationships_prev: typing.Optional[str]. The pagination token for the previous page of relationship results

            - relationships_limit: typing.Optional[int]. Limit total relationship values. Defaults to 50.

            - relationships_type: typing.Optional[Relationships]. Filter relationships to [relationship type](/sayari-library/ontology/relationships), e.g. director_of or has_shareholder

            - relationships_sort: typing.Optional[str]. Sorts relationships by As Of date or Shareholder percentage, e.g. date or -shares

            - relationships_start_date: typing.Optional[dt.date]. Filters relationships to after a date

            - relationships_end_date: typing.Optional[dt.date]. Filters relationships to before a date

            - relationships_min_shares: typing.Optional[int]. Filters relationships to greater than or equal to a Shareholder percentage

            - relationships_country: typing.Optional[typing.Union[Country, typing.Sequence[Country]]]. Filters relationships to a list of [countries](/sayari-library/ontology/enumerated-types#country)

            - relationships_arrival_country: typing.Optional[typing.Union[Country, typing.Sequence[Country]]]. Filters shipment relationships to a list of arrival [countries](/sayari-library/ontology/enumerated-types#country)

            - relationships_arrival_state: typing.Optional[str]. Filters shipment relationships to an arrival state

            - relationships_arrival_city: typing.Optional[str]. Filters shipment relationships to an arrival city

            - relationships_departure_country: typing.Optional[typing.Union[Country, typing.Sequence[Country]]]. Filters shipment relationships to a list of departure [countries](/sayari-library/ontology/enumerated-types#country)

            - relationships_departure_state: typing.Optional[str]. Filters shipment relationships to a departure state

            - relationships_departure_city: typing.Optional[str]. Filters shipment relationships to a departure city

            - relationships_partner_name: typing.Optional[str]. Filters shipment relationships to a trade partner name

            - relationships_partner_risk: typing.Optional[typing.Union[Risk, typing.Sequence[Risk]]]. Filters shipment relationships to a trade partner [risk tag](/sayari-library/ontology/enumerated-types#tag)

            - relationships_hs_code: typing.Optional[str]. Filters shipment relationships to an HS code

            - possibly_same_as_next: typing.Optional[str]. The pagination token for the next page of possibly same entities.

            - possibly_same_as_prev: typing.Optional[str]. The pagination token for the previous page of possibly same entities.

            - possibly_same_as_limit: typing.Optional[int]. Limit total possibly same as entities. Defaults to 100.

            - referenced_by_next: typing.Optional[str]. The pagination token for the next page of the entity's referencing records

            - referenced_by_prev: typing.Optional[str]. The pagination token for the previous page of the entity's referencing records

            - referenced_by_limit: typing.Optional[int]. Limit totals values returned for entity's referencing records. Defaults to 100.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.entity.get_entity(
            id="mGq1lpuqKssNWTjIokuPeA",
            attributes_name_limit=1,
            attributes_address_limit=1,
            attributes_country_limit=1,
            relationships_limit=1,
            possibly_same_as_limit=1,
            referenced_by_limit=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v1/entity/{jsonable_encoder(id)}"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "attributes.name.next": attributes_name_next,
                        "attributes.name.prev": attributes_name_prev,
                        "attributes.name.limit": attributes_name_limit,
                        "attributes.address.next": attributes_address_next,
                        "attributes.address.prev": attributes_address_prev,
                        "attributes.address.limit": attributes_address_limit,
                        "attributes.country.next": attributes_country_next,
                        "attributes.country.prev": attributes_country_prev,
                        "attributes.country.limit": attributes_country_limit,
                        "relationships.next": relationships_next,
                        "relationships.prev": relationships_prev,
                        "relationships.limit": relationships_limit,
                        "relationships.type": relationships_type,
                        "relationships.sort": relationships_sort,
                        "relationships.startDate": str(relationships_start_date)
                        if relationships_start_date is not None
                        else None,
                        "relationships.endDate": str(relationships_end_date)
                        if relationships_end_date is not None
                        else None,
                        "relationships.minShares": relationships_min_shares,
                        "relationships.country": relationships_country,
                        "relationships.arrivalCountry": relationships_arrival_country,
                        "relationships.arrivalState": relationships_arrival_state,
                        "relationships.arrivalCity": relationships_arrival_city,
                        "relationships.departureCountry": relationships_departure_country,
                        "relationships.departureState": relationships_departure_state,
                        "relationships.departureCity": relationships_departure_city,
                        "relationships.partnerName": relationships_partner_name,
                        "relationships.partnerRisk": relationships_partner_risk,
                        "relationships.hsCode": relationships_hs_code,
                        "possibly_same_as.next": possibly_same_as_next,
                        "possibly_same_as.prev": possibly_same_as_prev,
                        "possibly_same_as.limit": possibly_same_as_limit,
                        "referenced_by.next": referenced_by_next,
                        "referenced_by.prev": referenced_by_prev,
                        "referenced_by.limit": referenced_by_limit,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(GetEntityResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def entity_summary(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> EntitySummaryResponse:
        """
        The Entity Summary endpoint returns a smaller entity payload

        Parameters:
            - id: str. Unique identifier of the entity

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.entity.entity_summary(
            id="mGq1lpuqKssNWTjIokuPeA",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/entity_summary/{jsonable_encoder(id)}"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(EntitySummaryResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncEntityClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_entity(
        self,
        id: str,
        *,
        attributes_name_next: typing.Optional[str] = None,
        attributes_name_prev: typing.Optional[str] = None,
        attributes_name_limit: typing.Optional[int] = None,
        attributes_address_next: typing.Optional[str] = None,
        attributes_address_prev: typing.Optional[str] = None,
        attributes_address_limit: typing.Optional[int] = None,
        attributes_country_next: typing.Optional[str] = None,
        attributes_country_prev: typing.Optional[str] = None,
        attributes_country_limit: typing.Optional[int] = None,
        relationships_next: typing.Optional[str] = None,
        relationships_prev: typing.Optional[str] = None,
        relationships_limit: typing.Optional[int] = None,
        relationships_type: typing.Optional[Relationships] = None,
        relationships_sort: typing.Optional[str] = None,
        relationships_start_date: typing.Optional[dt.date] = None,
        relationships_end_date: typing.Optional[dt.date] = None,
        relationships_min_shares: typing.Optional[int] = None,
        relationships_country: typing.Optional[typing.Union[Country, typing.Sequence[Country]]] = None,
        relationships_arrival_country: typing.Optional[typing.Union[Country, typing.Sequence[Country]]] = None,
        relationships_arrival_state: typing.Optional[str] = None,
        relationships_arrival_city: typing.Optional[str] = None,
        relationships_departure_country: typing.Optional[typing.Union[Country, typing.Sequence[Country]]] = None,
        relationships_departure_state: typing.Optional[str] = None,
        relationships_departure_city: typing.Optional[str] = None,
        relationships_partner_name: typing.Optional[str] = None,
        relationships_partner_risk: typing.Optional[typing.Union[Risk, typing.Sequence[Risk]]] = None,
        relationships_hs_code: typing.Optional[str] = None,
        possibly_same_as_next: typing.Optional[str] = None,
        possibly_same_as_prev: typing.Optional[str] = None,
        possibly_same_as_limit: typing.Optional[int] = None,
        referenced_by_next: typing.Optional[str] = None,
        referenced_by_prev: typing.Optional[str] = None,
        referenced_by_limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetEntityResponse:
        """
        Retrieve an entity from the database based on the ID

        Parameters:
            - id: str. Unique identifier of the entity

            - attributes_name_next: typing.Optional[str]. The pagination token for the next page of attribute `name`.

            - attributes_name_prev: typing.Optional[str]. The pagination token for the previous page of attribute `name`.

            - attributes_name_limit: typing.Optional[int]. Limit total values returned for attribute `name`. Defaults to 100.

            - attributes_address_next: typing.Optional[str]. The pagination token for the next page of attribute `address`.

            - attributes_address_prev: typing.Optional[str]. The pagination token for the previous page of attribute `address`.

            - attributes_address_limit: typing.Optional[int]. Limit total values returned for attribute `address`. Defaults to 100.

            - attributes_country_next: typing.Optional[str]. The pagination token for the next page of attribute `country`.

            - attributes_country_prev: typing.Optional[str]. The pagination token for the previous page of attribute `country`.

            - attributes_country_limit: typing.Optional[int]. Limit total values returned for attribute `country`. Defaults to 100.

            - relationships_next: typing.Optional[str]. The pagination token for the next page of relationship results

            - relationships_prev: typing.Optional[str]. The pagination token for the previous page of relationship results

            - relationships_limit: typing.Optional[int]. Limit total relationship values. Defaults to 50.

            - relationships_type: typing.Optional[Relationships]. Filter relationships to [relationship type](/sayari-library/ontology/relationships), e.g. director_of or has_shareholder

            - relationships_sort: typing.Optional[str]. Sorts relationships by As Of date or Shareholder percentage, e.g. date or -shares

            - relationships_start_date: typing.Optional[dt.date]. Filters relationships to after a date

            - relationships_end_date: typing.Optional[dt.date]. Filters relationships to before a date

            - relationships_min_shares: typing.Optional[int]. Filters relationships to greater than or equal to a Shareholder percentage

            - relationships_country: typing.Optional[typing.Union[Country, typing.Sequence[Country]]]. Filters relationships to a list of [countries](/sayari-library/ontology/enumerated-types#country)

            - relationships_arrival_country: typing.Optional[typing.Union[Country, typing.Sequence[Country]]]. Filters shipment relationships to a list of arrival [countries](/sayari-library/ontology/enumerated-types#country)

            - relationships_arrival_state: typing.Optional[str]. Filters shipment relationships to an arrival state

            - relationships_arrival_city: typing.Optional[str]. Filters shipment relationships to an arrival city

            - relationships_departure_country: typing.Optional[typing.Union[Country, typing.Sequence[Country]]]. Filters shipment relationships to a list of departure [countries](/sayari-library/ontology/enumerated-types#country)

            - relationships_departure_state: typing.Optional[str]. Filters shipment relationships to a departure state

            - relationships_departure_city: typing.Optional[str]. Filters shipment relationships to a departure city

            - relationships_partner_name: typing.Optional[str]. Filters shipment relationships to a trade partner name

            - relationships_partner_risk: typing.Optional[typing.Union[Risk, typing.Sequence[Risk]]]. Filters shipment relationships to a trade partner [risk tag](/sayari-library/ontology/enumerated-types#tag)

            - relationships_hs_code: typing.Optional[str]. Filters shipment relationships to an HS code

            - possibly_same_as_next: typing.Optional[str]. The pagination token for the next page of possibly same entities.

            - possibly_same_as_prev: typing.Optional[str]. The pagination token for the previous page of possibly same entities.

            - possibly_same_as_limit: typing.Optional[int]. Limit total possibly same as entities. Defaults to 100.

            - referenced_by_next: typing.Optional[str]. The pagination token for the next page of the entity's referencing records

            - referenced_by_prev: typing.Optional[str]. The pagination token for the previous page of the entity's referencing records

            - referenced_by_limit: typing.Optional[int]. Limit totals values returned for entity's referencing records. Defaults to 100.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.entity.get_entity(
            id="mGq1lpuqKssNWTjIokuPeA",
            attributes_name_limit=1,
            attributes_address_limit=1,
            attributes_country_limit=1,
            relationships_limit=1,
            possibly_same_as_limit=1,
            referenced_by_limit=1,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v1/entity/{jsonable_encoder(id)}"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "attributes.name.next": attributes_name_next,
                        "attributes.name.prev": attributes_name_prev,
                        "attributes.name.limit": attributes_name_limit,
                        "attributes.address.next": attributes_address_next,
                        "attributes.address.prev": attributes_address_prev,
                        "attributes.address.limit": attributes_address_limit,
                        "attributes.country.next": attributes_country_next,
                        "attributes.country.prev": attributes_country_prev,
                        "attributes.country.limit": attributes_country_limit,
                        "relationships.next": relationships_next,
                        "relationships.prev": relationships_prev,
                        "relationships.limit": relationships_limit,
                        "relationships.type": relationships_type,
                        "relationships.sort": relationships_sort,
                        "relationships.startDate": str(relationships_start_date)
                        if relationships_start_date is not None
                        else None,
                        "relationships.endDate": str(relationships_end_date)
                        if relationships_end_date is not None
                        else None,
                        "relationships.minShares": relationships_min_shares,
                        "relationships.country": relationships_country,
                        "relationships.arrivalCountry": relationships_arrival_country,
                        "relationships.arrivalState": relationships_arrival_state,
                        "relationships.arrivalCity": relationships_arrival_city,
                        "relationships.departureCountry": relationships_departure_country,
                        "relationships.departureState": relationships_departure_state,
                        "relationships.departureCity": relationships_departure_city,
                        "relationships.partnerName": relationships_partner_name,
                        "relationships.partnerRisk": relationships_partner_risk,
                        "relationships.hsCode": relationships_hs_code,
                        "possibly_same_as.next": possibly_same_as_next,
                        "possibly_same_as.prev": possibly_same_as_prev,
                        "possibly_same_as.limit": possibly_same_as_limit,
                        "referenced_by.next": referenced_by_next,
                        "referenced_by.prev": referenced_by_prev,
                        "referenced_by.limit": referenced_by_limit,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(GetEntityResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def entity_summary(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> EntitySummaryResponse:
        """
        The Entity Summary endpoint returns a smaller entity payload

        Parameters:
            - id: str. Unique identifier of the entity

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.entity.entity_summary(
            id="mGq1lpuqKssNWTjIokuPeA",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/entity_summary/{jsonable_encoder(id)}"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(EntitySummaryResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...base_types.types.paginated_response import PaginatedResponse
from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import pydantic_v1
from .status_data import StatusData


class StatusInfo(PaginatedResponse):
    """
    The status of an entity. This attribute is used to indicate details such as registration, operating, or liquidation status as well as an entity's license or sole proprietorship status.
    """

    data: typing.List[StatusData]
    next: typing.Optional[typing.Any] = None
    offset: typing.Optional[int] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import pydantic_v1
from .save_entity_response_data import SaveEntityResponseData


class SaveEntityResponse(pydantic_v1.BaseModel):
    """
    from sayari import SaveEntityResponse, SaveEntityResponseData

    SaveEntityResponse(
        data=SaveEntityResponseData(
            type="entity",
            id="Ywk6qw",
            project="GNJbkG",
            label="HOME DEPOT, U. S. A., INC.",
            created="2024-03-12 16:12:38.528362+00",
            updated="2024-03-12 16:12:38.528362+00",
            updated_by="auth0|5e45bd8caccd890e68147513",
            version=0,
            entity_id="n9SA4T_33tDmLtS8BhC6Zg",
            tag_ids=[],
            case_status="not_assigned",
        ),
    )
    """

    data: SaveEntityResponseData

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .business_purpose import BusinessPurpose
from .buyer_search_response import BuyerSearchResponse
from .data_source import DataSource
from .hs_code import HsCode
from .hs_code_info import HsCodeInfo
from .monetary_value import MonetaryValue
from .shipment import Shipment
from .shipment_address import ShipmentAddress
from .shipment_country import ShipmentCountry
from .shipment_identifier import ShipmentIdentifier
from .shipment_metadata import ShipmentMetadata
from .shipment_search_response import ShipmentSearchResponse
from .source_or_destination_entity import SourceOrDestinationEntity
from .supplier_metadata import SupplierMetadata
from .supplier_or_buyer import SupplierOrBuyer
from .supplier_search_response import SupplierSearchResponse
from .trade_filter_list import TradeFilterList
from .weight import Weight

__all__ = [
    "BusinessPurpose",
    "BuyerSearchResponse",
    "DataSource",
    "HsCode",
    "HsCodeInfo",
    "MonetaryValue",
    "Shipment",
    "ShipmentAddress",
    "ShipmentCountry",
    "ShipmentIdentifier",
    "ShipmentMetadata",
    "ShipmentSearchResponse",
    "SourceOrDestinationEntity",
    "SupplierMetadata",
    "SupplierOrBuyer",
    "SupplierSearchResponse",
    "TradeFilterList",
    "Weight",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import pydantic_v1
from ...shared_types.types.core_entity import CoreEntity
from .project_entity_upstream import ProjectEntityUpstream
from .psa_summary import PsaSummary


class ProjectEntity(pydantic_v1.BaseModel):
    id: str
    project: str
    label: str = pydantic_v1.Field()
    """
    Entity label (display name).
    """

    created: str
    updated: str
    updated_by: str
    version: int = pydantic_v1.Field()
    """
    Will be 0.
    """

    type: typing.Literal["entity"]
    entity_id: str = pydantic_v1.Field()
    """
    Entity ID.
    """

    tag_ids: typing.List[str]
    case_status: str
    match_strength: typing.Any
    shipped_hs_codes: typing.List[str] = pydantic_v1.Field()
    """
    HS codes shipped by the entity.
    """

    received_hs_codes: typing.List[str] = pydantic_v1.Field()
    """
    HS codes received by the entity.
    """

    upstream: ProjectEntityUpstream
    summary: CoreEntity
    psa: typing.Optional[PsaSummary] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...base_types.types.paginated_response import PaginatedResponse
from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import pydantic_v1
from .supplier_or_buyer import SupplierOrBuyer


class SupplierSearchResponse(PaginatedResponse):
    """
    OK
    ---
    from sayari import (
        HsCode,
        QualifiedCount,
        SourceCountInfo,
        SupplierMetadata,
        SupplierOrBuyer,
        SupplierSearchResponse,
    )

    SupplierSearchResponse(
        offset=0,
        limit=1,
        size=QualifiedCount(
            count=5541,
            qualifier="eq",
        ),
        next=True,
        data=[
            SupplierOrBuyer(
                id="LvKN1AA74GQ289vydvYgtg",
                label="RedbusAvto OU",
                degree=2523,
                entity_url="/v1/entity/LvKN1AA74GQ289vydvYgtg",
                pep=False,
                psa_count=0,
                sanctioned=False,
                closed=False,
                type="company",
                identifiers=[],
                addresses=[],
                countries=[],
                relationship_count={"shipper_of": 2522, "ships_to": 1},
                source_count={
                    "96c06a5a03b61b91324c7e05b3114fb6": SourceCountInfo(
                        count=2522,
                        label="Ukraine Imports & Exports (January 2023 - Present)",
                    )
                },
                risk={},
                user_attribute_count={},
                user_record_count=0,
                user_related_entities_count=0,
                user_relationship_count={},
                related_entities_count=2523,
                attribute_count={"name": 1},
                metadata=SupplierMetadata(
                    latest_shipment_date="2023-10-27",
                    shipments=2522,
                    hs_codes=[
                        HsCode(
                            key="870323",
                            doc_count=1292,
                            value="Vehicles; with only spark-ignition internal combustion reciprocating piston engine, cylinder capacity over 1500 but not over 3000cc",
                            value_simple="Cars & Passenger Vehicles",
                        ),
                        HsCode(
                            key="8703239013",
                            doc_count=1288,
                            value="Vehicles; with only spark-ignition internal combustion reciprocating piston engine, cylinder capacity over 1500 but not over 3000cc",
                            value_simple="Cars & Passenger Vehicles",
                        ),
                        HsCode(
                            key="870332",
                            doc_count=509,
                            value="Vehicles; with only compression-ignition internal combustion piston engine (diesel or semi-diesel), cylinder capacity over 1500 but not over 2500cc",
                            value_simple="Cars & Passenger Vehicles",
                        ),
                        HsCode(
                            key="8703329030",
                            doc_count=508,
                            value="Vehicles; with only compression-ignition internal combustion piston engine (diesel or semi-diesel), cylinder capacity over 1500 but not over 2500cc",
                            value_simple="Cars & Passenger Vehicles",
                        ),
                        HsCode(
                            key="870322",
                            doc_count=472,
                            value="Vehicles; with only spark-ignition internal combustion piston engine, cylinder capacity over 1000 but not over 1500cc",
                            value_simple="Cars & Passenger Vehicles",
                        ),
                    ],
                ),
            )
        ],
    )
    """

    offset: int
    next: bool
    data: typing.List[SupplierOrBuyer]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import pydantic_v1
from .resource_notification_data import ResourceNotificationData


class ResourceNotificationsResponse(pydantic_v1.BaseModel):
    """
    OK
    ---
    from sayari import ResourceNotificationData, ResourceNotificationsResponse

    ResourceNotificationsResponse(
        offset=0,
        limit=20,
        next=False,
        data=[
            ResourceNotificationData(
                saved_resource_id="03ePyj",
                project_id="0oZnoG",
                entity_id="wxwqZshCF4trlrmOa2eu9w",
                type="risk",
                field="forced_labor_sheffield_hallam_university_reports_origin_subtier",
                value="3",
                date="2024-02-15T00:00:00.000Z",
            )
        ],
    )
    """

    offset: int
    limit: int
    next: bool
    data: typing.List[ResourceNotificationData]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime
from ...core.pydantic_utilities import pydantic_v1
from ...generated_types.types.relationships import Relationships
from .relationship_info import RelationshipInfo


class RelationshipData(pydantic_v1.BaseModel):
    target: EntityDetails
    types: typing.Dict[Relationships, typing.List[RelationshipInfo]] = pydantic_v1.Field()
    """
    Additional information for each [relationship type](/sayari-library/ontology/relationships).
    """

    dates: typing.List[str]
    first_observed: typing.Optional[str] = None
    former: typing.Optional[bool] = None
    last_observed: typing.Optional[str] = None
    start_date: typing.Optional[str] = None
    end_date: typing.Optional[str] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}


from .entity_details import EntityDetails  # noqa: E402

RelationshipData.update_forward_refs()



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import pydantic_v1
from ..core.remove_none_from_dict import remove_none_from_dict
from ..core.request_options import RequestOptions
from ..shared_errors.errors.bad_request import BadRequest
from ..shared_errors.errors.internal_server_error import InternalServerError
from ..shared_errors.errors.method_not_allowed import MethodNotAllowed
from ..shared_errors.errors.not_found import NotFound
from ..shared_errors.errors.rate_limit_exceeded import RateLimitExceeded
from ..shared_errors.errors.unauthorized import Unauthorized
from ..shared_errors.types.bad_request_response import BadRequestResponse
from ..shared_errors.types.internal_server_error_response import InternalServerErrorResponse
from ..shared_errors.types.method_not_allowed_response import MethodNotAllowedResponse
from ..shared_errors.types.not_found_response import NotFoundResponse
from ..shared_errors.types.rate_limit_response import RateLimitResponse
from ..shared_errors.types.unauthorized_response import UnauthorizedResponse
from .types.save_entity_request import SaveEntityRequest
from .types.save_entity_response import SaveEntityResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ResourceClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def save_entity(
        self, *, request: SaveEntityRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> SaveEntityResponse:
        """
        <Callout intent="warning">This endpoint is in beta and is subject to change. It is provided for early access and testing purposes only.</Callout> Save an entity to a project.

        Parameters:
            - request: SaveEntityRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from sayari import SaveEntityRequest
        from sayari.client import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.resource.save_entity(
            request=SaveEntityRequest(
                type="entity",
                project="GNJbkG",
                entity_id="Zk0qOaM2SSYg_ZhsljykMQ",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/resource/entity"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(SaveEntityResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncResourceClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def save_entity(
        self, *, request: SaveEntityRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> SaveEntityResponse:
        """
        <Callout intent="warning">This endpoint is in beta and is subject to change. It is provided for early access and testing purposes only.</Callout> Save an entity to a project.

        Parameters:
            - request: SaveEntityRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from sayari import SaveEntityRequest
        from sayari.client import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        await client.resource.save_entity(
            request=SaveEntityRequest(
                type="entity",
                project="GNJbkG",
                entity_id="Zk0qOaM2SSYg_ZhsljykMQ",
            ),
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/resource/entity"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(SaveEntityResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequest(pydantic_v1.parse_obj_as(BadRequestResponse, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise Unauthorized(pydantic_v1.parse_obj_as(UnauthorizedResponse, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFound(pydantic_v1.parse_obj_as(NotFoundResponse, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowed(pydantic_v1.parse_obj_as(MethodNotAllowedResponse, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise RateLimitExceeded(pydantic_v1.parse_obj_as(RateLimitResponse, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(
                pydantic_v1.parse_obj_as(InternalServerErrorResponse, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from .resource_type import ResourceType
from .save_entity_request import SaveEntityRequest
from .save_entity_response import SaveEntityResponse
from .save_entity_response_data import SaveEntityResponseData

__all__ = ["ResourceType", "SaveEntityRequest", "SaveEntityResponse", "SaveEntityResponseData"]



# This file was auto-generated by Fern from our API Definition.

import typing

SearchField = typing.Union[
    typing.Literal["name", "identifier", "address", "business_purpose", "date_of_birth", "contact"], typing.Any
]
